-- Instructions display

local MAXDISPLAYTIME = 2*TICRATE

local FADETIME = 7

rawset(_G, "WW_Instruction", function(player, ...)
	if player.spectator then return end
	
	player.ww_instruction = { text = {...}, timer = MAXDISPLAYTIME }
end)

rawset(_G, "WW_InstructionAll", function(...)
	for player in players.iterate do
		WW_Instruction(player, ...)
	end
end)


rawset(_G, "WW_InstructionTeam", function(team, ...)
	for player in players.iterate do
		if player.ww_team == team then WW_Instruction(player, ...) end
	end
end)

addHook("MapLoad", function()
	for player in players.iterate do
		player.ww_instruction = nil
	end
end)

addHook("ThinkFrame", function()
	for player in players.iterate do
		if player.ww_instruction then
			player.ww_instruction.timer = $ - 1
			
			if player.ww_instruction.timer == 0 then
				player.ww_instruction = nil
			end
		end
	end
end)

local function rescale(value, oldmin, oldmax, newmin, newmax)
    return newmin + FixedMul(FixedDiv(value - oldmin, oldmax - oldmin), newmax - newmin)
end

hud.add(function(v, p, c)
	if not p.ww_instruction then return end
	
	local i = p.ww_instruction
	
	if splitscreen < 2 then -- Be fancy	
		local x, y = 0, (200/(3 - c.pnum))-20-(#i.text*10)
		local flags = V_HUDTRANS
		
		if splitscreen == 1 and c.pnum == 1 then
			flags = $|V_SNAPTOBOTTOM
		end
		
		-- Fade in
		if i.timer <= FADETIME then
			x = rescale(i.timer<<FRACBITS, FADETIME<<FRACBITS, 0, 0, 320<<FRACBITS)>>FRACBITS
			x = -x
		elseif i.timer >= MAXDISPLAYTIME-FADETIME then
			x = rescale((MAXDISPLAYTIME-i.timer)<<FRACBITS, 0, FADETIME<<FRACBITS, 320<<FRACBITS, 0)>>FRACBITS
		else
			x = 0
		end
		
		v.drawFill(x, y, 320, #i.text*10+2, 240|flags)
		
		for num, str in ipairs(i.text) do
			v.drawString(x+320/2, y + 10*(num-1) + 2, i.text[num], flags|V_ALLOWLOWERCASE, "center")
		end
	else -- Be minimal
		local splitnum = c.pnum - 1
		
		local x, y = 0, 0
		local flags = V_HUDTRANS
		
		if splitnum % 2 == 0 then
			x = 320/4
			flags = $|V_SNAPTOLEFT
		else
			x = 3*320/4
		end
		
		if splitnum < 2 then
			y = 200/2
		else
			y = 200
			flags = $|V_SNAPTOBOTTOM
		end
		
		y = y - 10 -(#i.text*8)
		
		for num, str in ipairs(i.text) do
			local w = v.stringWidth(i.text[num], flags|V_ALLOWLOWERCASE, "small")
			v.drawString(x - w/2, y + 8*(num-1) + 2, i.text[num], flags|V_ALLOWLOWERCASE, "small")
		end
	end
end)