-- Scoring through lines. Award is given after minigame ends via linedef executor

local scorelines = {}

-- TODO - change names, rn they might conflict with wario deluxe
local SCORING_LINES = {
    { name = "WADEP1", maxscore = 2 },
    { name = "WADEP2", maxscore = 2 },
    { name = "WADEP3", maxscore = 2 },
    { name = "WADEP4", maxscore = 4 },
    { name = "WADEP5", maxscore = 5 },
    { name = "WADEP6", maxscore = 6 },
    { name = "WADEP7", maxscore = 7 },
    { name = "WADESOLO", maxscore = 1 },
    { name = "WADEEND", maxscore = 3 },
}

local function getLinedefScore(name)
    if scorelines[name] == nil then error(string.format("Unknown scoring linedef %q", name)) end
    
    local score = scorelines[name].score
    scorelines[name].score = max($ - 1, 1)
    
    return score
end

local function initScoreLines()
	scorelines = {}

	for i = 1, #SCORING_LINES do
		local line = SCORING_LINES[i]

		scorelines[line.name] = { score = line.maxscore, maxscore = line.maxscore }
	end
end

for i = 1, #SCORING_LINES do
	local line = SCORING_LINES[i]
	
	addHook("LinedefExecute", function(_, pmo, _)
		local player = pmo.player
		
		if not (player and player.valid and not player.ww_delayed_points) then return end

		WW_AddPointsDelayed(player, getLinedefScore(line.name))
	end, line.name)
end


addHook("NetVars", function(net)
	scorelines = net($)
end)

addHook("MapLoad", function(mapid)
	if G_BuildMapName(mapid) ~= "MAPWR" then return end
	
	initScoreLines()
end)