local MAP = "MAPWR"

-- Time before exitlevel
local EXITLEVEL_TIMER = 10*TICRATE

-- Time before show forcefully make everyone finish
local ENDING_TIMER = 20*TICRATE

-- Funny syntax sugar
local function enum(l)
    local e = {}

    for i = 0, #l-1 do
        e[l[i+1]] = i
    end
    
    return e
end

local LevelState = enum {
    "OnGoing", -- No one finished yet
    "Ending", -- One player finished, counting down and moving onto next state
    "AllDone", -- Show results and exitlevel
}

local scoring = {
    running = false,
    state = LevelState.OnGoing,
    timer = 0,
}

-- TODO
-- ...or not?
local function addPointsAnimation(player, num)
    K_SpawnBattlePoints(player, nil, num)
end

local function addPoints(player, num)
    if num == nil then num = 1 end
    
    player.marescore = $ + num
    addPointsAnimation(player, num)
end

local function resetGlobalState()    
    scoring.running = false
    scoring.state = LevelState.OnGoing
    scoring.timer = 0
end

local function updateHud()
    if scoring.running then
        hud.disable("position")
        hud.disable("gametypeinfo")
    else
        hud.enable("position")
        hud.enable("gametypeinfo")
    end
end

local function countPlayers()
    local in_game = 0
    local exiting = 0
    
    for player in players.iterate do
        if player.spectator or (player.pflags & PF_TIMEOVER) then continue end
        
        if player.exiting then
            exiting = $ + 1
        else
            in_game = $ + 1
        end
    end
    
    return in_game, exiting
end

local function updateLevelState()
    if scoring.state == LevelState.AllDone then return end
    
    local in_game, exiting = countPlayers()
    
    if in_game == 0 then
        -- Everyone finished, start final countdown and show results
        scoring.state = LevelState.AllDone
        scoring.timer = EXITLEVEL_TIMER
    elseif exiting > 0 then
        -- Some are finished, start countdown before forcefully switching to AllDone state
        scoring.state = LevelState.Ending
        scoring.timer = ENDING_TIMER
    end
end

local function updateRealtimes()
    local playerlist = {}
    
    for player in players.iterate do
        if not player.spectator then table.insert(playerlist, player) end
    end
    
    table.sort(playerlist, function(p1, p2) return p1.marescore > p2.marescore end)
    
    for i = 1, #playerlist do
        playerlist[i].realtime = i*TICRATE*60
    end
end

local function forceEnd()
    scoring.state = LevelState.AllDone
    
    for player in players.iterate do
        player.exiting = 1
        player.pflags = $ & (~PF_TIMEOVER)
        player.lives = 1
    end
    
    updateRealtimes()
end

local function addPointsDelayed(player, points)
	player.ww_delayed_points = ($ or 0) + points
end

local function giveDelayedPoints()
	for player in players.iterate do
		if player.ww_delayed_points then
			WW_AddPoints(player, player.ww_delayed_points)
			player.ww_delayed_points = nil
		end
	end
end

addHook("LinedefExecute", giveDelayedPoints, "WW_GIVEPOINTS")

local cv_colorizedhud, cv_colorizedhudcolor
local function lookupCvars()
    if cv_colorizedhud then return end

    cv_colorizedhud = CV_FindVar("colorizedhud")
    cv_colorizedhudcolor = CV_FindVar("colorizedhudcolor")
end

local function getHudColor(p)
    if not (cv_colorizedhud and cv_colorizedhud.value) then return end
    
    if cv_colorizedhudcolor.value then return cv_colorizedhudcolor.value end
    
    return p.skincolor
end

addHook("MapLoad", lookupCvars)
addHook("NetVars", lookupCvars)

local lapn_patch, lapn_patch_clr
local function hudScore(v, p)
    if not lapn_patch then lapn_patch = v.cachePatch("K_STLAPN") end
  
    local splitnum = p.splitscreenindex

    local x, y

    local flags = V_HUDTRANS | V_SNAPTOLEFT | V_SNAPTOBOTTOM

    if splitscreen > 0 then
        if splitscreen == 1 then
            x = 20
            y = ((splitnum == 0) and 100 or 200) - 40
        else
            x = ((splitnum == 0) or splitnum == 2) and 20 or 180
            y = (((splitnum == 0) or splitnum == 1) and 100 or 200) - 40
        end
    else
        x = 20
        y = 200 - 38
    end

    local colormap
    local patch = lapn_patch

    if cv_colorizedhud and cv_colorizedhud.value then
        if not lapn_patch_clr then lapn_patch_clr = v.cachePatch("K_SCLAPN") end

        patch = lapn_patch_clr

        local color = getHudColor(p)

        colormap = v.getColormap(p.mo.skin, color)
    end

    if splitscreen < 2 then
        v.draw(x, y, patch, flags, colormap)
    end

    v.drawString(x+14, y+6, string.format("%02d pts", p.marescore), flags, splitscreen > 1 and "small" or "thin")
end

local cv_highresportrait
local function hudScoreboard(v)
	if cv_highresportrait == nil then
		cv_highresportrait = CV_FindVar("highresportrait") or false
	end

    v.fadeScreen(65280, 16)

	v.drawString(160, 12, "* SCORES *", 0, "center")
    v.drawFill(1, 26, 318, 1, 0)
    
    
    local playerlist = {}
    
    for player in players.iterate do
        if not player.spectator then table.insert(playerlist, player) end
    end
    
    table.sort(playerlist, function(p1, p2) return p1.marescore > p2.marescore end)
    
    local pos = 1
    local previous_score
    for i = 1, #playerlist do
        local player = playerlist[i]
        local name = player.name or "NULL"
        local y = 23 + i*10
		
		local skin = player.mo and player.mo.skin
		
		if skin then
			local patch
			local scale = FRACUNIT/2
			
			if cv_highresportrait and cv_highresportrait.value then
				patch = v.cachePatch(skins[skin].facewant)
				scale = scale/2
			else
				patch = v.cachePatch(skins[skin].facerank)
			end
			
			v.drawScaled(91<<FRACBITS, y<<FRACBITS, scale, patch, 0, v.getColormap(skin, player.skincolor))
		end

        v.drawString(100, y, name, V_6WIDTHSPACE|V_ALLOWLOWERCASE, "thin")

        local points = player.marescore
        if previous_score ~= nil and points < previous_score then
            pos = pos + 1
        end
        
        previous_score = points

        v.drawString(40, y, ""..pos..".", V_6WIDTHSPACE|V_ALLOWLOWERCASE, "thin")
        v.drawString(280, y, ""..points.." pts", V_6WIDTHSPACE|V_ALLOWLOWERCASE, "thin-right")
    end
end

addHook("MapLoad", function(mapid)
    scoring.running = false

    if G_BuildMapName(mapid) == MAP then
        resetGlobalState()
        scoring.running = true
        
        -- Not sure if i need to do this...
        for player in players.iterate do
            player.marescore = 0
        end
		
		hud.disable("minirankings")
		if minrankplus then minrankplus.enabled = false end
    else
		hud.enable("minirankings")
		if minrankplus then minrankplus.enabled = true end
	end
    
    updateHud()
end)

addHook("NetVars", function(net)
    scoring = net($)
    
    updateHud()
end)

addHook("ThinkFrame", function()
    if not scoring.running then return end
    if leveltime < 2 then return end
    
    updateLevelState()
    
    scoring.timer = max(0, $-1)
    
    if scoring.state == LevelState.Ending then
        if scoring.timer == 0 then
            forceEnd()
            return
        end
    elseif scoring.state == LevelState.AllDone then
        updateRealtimes()
        
        -- Just so we don't exitlevel too early
        K_SetExitCountdown(scoring.timer)
    
        if scoring.timer == 0 then
            G_ExitLevel()
        end
    end
end)

hud.add(function(v, p)
    if not (scoring.running and not p.spectator) then return end
    
    if (p.exiting and splitscreen == 0) or scoring.state == LevelState.AllDone then
        hudScoreboard(v)
        return
    end
    
    hudScore(v, p)
end)

rawset(_G, "WW_StateEnum", LevelState)
rawset(_G, "WW_GetGameState", function() return scoring end)
rawset(_G, "WW_AddPoints", addPoints)
rawset(_G, "WW_AddPointsDelayed", addPointsDelayed)
rawset(_G, "WW_ForceEnd", forceEnd)