local TELEPORT_LINEDEFS = {
    LINETLPT = true,
    CIRCTLPT = true,
    LINETLPTALL = true,
    CIRCTLPTALL = true,
	LINETLPTTEAM = true,
}

local teleport_sectors = {} -- tag : sector_num

local function findSectorNumByTag(tag)
    for i = 1, #sectors do
        if sectors[i].tag == tag then return i end
    end
end

addHook("MapLoad", function()
    teleport_sectors = {}

    for i = 0, #lines-1 do
        if lines[i].special == 443 and TELEPORT_LINEDEFS[lines[i].frontside.text] then
            teleport_sectors[lines[i].tag] = findSectorNumByTag(lines[i].tag)
        end
    end
end)

addHook("NetVars", function(net)
    teleport_sectors = net($)
end)

local function lineTeleport(line, mo)
    if not (mo and mo.valid) then return end

    local destsectornum = teleport_sectors[line.tag]

    if destsectornum == nil then error("LINETLPT: Destination sector not found for tag "..line.tag) end

    local destsector = sectors[destsectornum]

    local dest

    for thing in destsector.thinglist() do
        if thing.type == MT_TELEPORTMAN then -- man...
            dest = thing
            break
        end
    end

    if not dest then error("LINETLPT: Destination thing not found for tag "..line.tag) end

    local num = 0

    if mo.player then num = #mo.player + 2 end

    local mul = (num % 2 == 0) and 1 or -1

    P_SetOrigin(mo, dest.x + P_ReturnThrustX(dest.angle + ANGLE_90 * mul, 64*mapobjectscale*(num/2)), dest.y + P_ReturnThrustY(dest.angle + ANGLE_90 * mul, 64*mapobjectscale*(num/2)), dest.z)
    mo.angle = dest.angle

    mo.momx = 0
    mo.momy = 0
    mo.momz = 0

    mo.reactiontime = TICRATE/2
	
	if mo.player then
		mo.player.kartstuff[k_driftboost] = 0
	end
end

local function circleTeleport(line, mo)
    if not (mo and mo.valid) then return end

    local destsectornum = teleport_sectors[line.tag]

    if destsectornum == nil then error("LINETLPT: Destination sector not found for tag "..line.tag) end

    local destsector = sectors[destsectornum]

    local dest

    for thing in destsector.thinglist() do
        if thing.type == MT_TELEPORTMAN then -- man...
            dest = thing
            break
        end
    end

    if not dest then error("LINETLPT: Destination thing not found for tag "..line.tag) end

	local radius = (line.frontside.textureoffset>>FRACBITS) or 708

    local num = 0

    if mo.player then num = #mo.player end

    P_SetOrigin(mo, dest.x + P_ReturnThrustX(ANG1 * (360 / #players) * num, radius*mapobjectscale), dest.y + P_ReturnThrustY(ANG1 * (360 / #players) * num, radius*mapobjectscale), dest.z)
    mo.angle = ANGLE_180 + (ANG1 * (360 / #players) * num)

    mo.momx = 0
    mo.momy = 0
    mo.momz = 0

    mo.reactiontime = TICRATE/2
	
	
	if mo.player then
		mo.player.kartstuff[k_driftboost] = 0
	end
end

addHook("LinedefExecute", function(line, mo, sec)
    lineTeleport(line, mo)
end, "LINETLPT")

addHook("LinedefExecute", function(line, mo, sec)
    circleTeleport(line, mo)
end, "CIRCTLPT")


addHook("LinedefExecute", function(line, mo, sec)
    for player in players.iterate do
        if player.mo then
            lineTeleport(line, player.mo)
        end
    end
end, "LINETLPTALL")

addHook("LinedefExecute", function(line, mo, sec)
    for player in players.iterate do
        if player.mo then
            circleTeleport(line, player.mo)
        end
    end
end, "CIRCTLPTALL")

addHook("LinedefExecute", function(line, mo, sec)
    for player in players.iterate do
        if player.mo and player.ww_team == (line.frontside.textureoffset>>FRACBITS) then
            lineTeleport(line, player.mo)
        end
    end
end, "LINETLPTTEAM")