-- Shared stuff for team modes

local teams = false

rawset(_G, "WW_TEAM_BLUE", 1)
rawset(_G, "WW_TEAM_RED", 2)

rawset(_G, "WW_TEAM_NAMES",  {
	[WW_TEAM_BLUE] = "blue",
	[WW_TEAM_RED] = "red",
})

rawset(_G, "WW_TEAM_COLORS", {
	[WW_TEAM_BLUE] = SKINCOLOR_BLUE,
	[WW_TEAM_RED] = SKINCOLOR_RED,
})

local function restoreColor(p)
	if not p.ww_save_color then return end
	
	p.skincolor = p.ww_save_color
	if p.mo then p.mo.color = p.skincolor end
	
	p.ww_save_color = nil
end

local function replaceColor(p, replace)
	if not p.ww_save_color then
		p.ww_save_color = p.skincolor
	end

	if not replace then
		restoreColor(p)
		return
	end

	p.skincolor = replace
	if p.mo then p.mo.color = replace end
end

rawset(_G, "WW_AssignTeams", function()
	teams = true

	local player_list = {}
	
	for player in players.iterate do
		if not player.spectator then
			player.ww_team = nil
			
			table.insert(player_list, player)
		end
	end
	
	-- Half of players are team 1
	for i = 1, #player_list/2 do
		local p = table.remove(player_list, P_RandomRange(1, #player_list))
		
		p.ww_team = WW_TEAM_BLUE
	end
	
	-- Other half are team 2
	for _, p in ipairs(player_list) do
		p.ww_team = WW_TEAM_RED
	end
end)

rawset(_G, "WW_ResetTeams", function()
	if not teams then return end
	
	teams = false
	
	for player in players.iterate do
		player.ww_team = nil
		restoreColor(player)
	end
end)

rawset(_G, "WW_AddPointsTeam", function(team, numpoints)
	if not teams then return end
	
	for player in players.iterate do
		if player.ww_team == team then
			WW_AddPoints(player, numpoints)
		end
	end
end)

rawset(_G, "WW_AddPointsDelayedTeam", function(team, numpoints)
	if not teams then return end
	
	for player in players.iterate do
		if player.ww_team == team then
			WW_AddPoints(player, numpoints)
		end
	end
end)

addHook("ThinkFrame", function()
	if not teams then return end

	for player in players.iterate do
		if not player.spectator then
			replaceColor(player, WW_TEAM_COLORS[player.ww_team])
		end
	end
end)

addHook("NetVars", function(net)
	teams = net($)
end)

-- Failsafe
addHook("MapLoad", WW_ResetTeams)